@@ -12,9 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from keystoneclient.auth.identity import v2 as v2_client
-from keystoneclient.auth.identity import v3 as v3_client
-from keystoneclient import session
+from keystoneauth1.identity import v2 as v2_client
+from keystoneauth1.identity import v3 as v3_client
+from keystoneauth1 import session
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -61,7 +61,8 @@ def get_session():
         try:
             kc = client.Password(**kwargs)
             _SESSION = session.Session(
-                auth=kc, verify=not cfg.CONF.keystone_authtoken.insecure)
+                auth=kc, verify=(cfg.CONF.keystone_authtoken.cafile or
+                                 not cfg.CONF.keystone_authtoken.insecure))
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE("Error creating Keystone session."))
