@@ -60,7 +60,8 @@ class BaseDatabaseTask(task.Task):
         try:
             self.amp_health_repo.delete(db_apis.get_session(),
                                         amphora_id=amphora_id)
-        except sqlalchemy.orm.exc.NoResultFound:
+        except (sqlalchemy.orm.exc.NoResultFound,
+                sqlalchemy.orm.exc.UnmappedInstanceError):
             LOG.debug('No existing amphora health record to delete '
                       'for amphora: %s, skipping.', amphora_id)
 
@@ -74,7 +75,8 @@ class BaseDatabaseTask(task.Task):
             self.amp_health_repo.update(db_apis.get_session(),
                                         amphora_id=amphora_id,
                                         busy=True)
-        except sqlalchemy.orm.exc.NoResultFound:
+        except (sqlalchemy.orm.exc.NoResultFound,
+                sqlalchemy.orm.exc.UnmappedInstanceError):
             LOG.debug('No existing amphora health record to mark busy '
                       'for amphora: %s, skipping.', amphora_id)
 
@@ -428,6 +430,14 @@ class MapLoadbalancerToAmphora(BaseDatabaseTask):
 
         return amp.id
 
+    def revert(self, result, loadbalancer_id, *args, **kwargs):
+        LOG.warning(_LW("Reverting Amphora allocation for the load "
+                        "balancer %s in the database."), loadbalancer_id)
+
+        self.loadbalancer_repo.update(db_apis.get_session(),
+                                      loadbalancer_id,
+                                      provisioning_status=constants.ERROR)
+
 
 class _MarkAmphoraRoleAndPriorityInDB(BaseDatabaseTask):
     """Alter the amphora role in DB."""
