@@ -11,29 +11,30 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
 import hashlib
 import json
+import os
+import random
+import stat
 import subprocess
 
 import mock
 import netifaces
 import six
+import six.moves.builtins as builtins
 
 from octavia.amphorae.backends.agent import api_server
 from octavia.amphorae.backends.agent.api_server import certificate_update
 from octavia.amphorae.backends.agent.api_server import server
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.common import constants as consts
+from octavia.common import utils as octavia_utils
+from octavia.tests.common import utils as test_utils
 import octavia.tests.unit.base as base
 
 RANDOM_ERROR = 'random error'
 OK = dict(message='OK')
 
-BUILTINS = '__builtin__'
-if six.PY3:
-    BUILTINS = 'builtins'
-
 
 class ServerTestCase(base.TestCase):
     app = None
@@ -49,45 +50,63 @@ class ServerTestCase(base.TestCase):
     @mock.patch('os.remove')
     def test_haproxy(self, mock_remove, mock_subprocess, mock_rename,
                      mock_makedirs, mock_exists):
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
         mock_exists.return_value = True
+        file_name = '/var/lib/octavia/123/haproxy.cfg.new'
         m = mock.mock_open()
 
         # happy case upstart file exists
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
             rv = self.app.put('/' + api_server.VERSION +
-                              '/listeners/123/haproxy', data='test')
+                              '/listeners/amp_123/123/haproxy',
+                              data='test')
+            mode = stat.S_IRUSR | stat.S_IWUSR
+            mock_open.assert_called_with(file_name, flags, mode)
+            mock_fdopen.assert_called_with(123, 'w')
             self.assertEqual(202, rv.status_code)
-            m.assert_called_once_with(
-                '/var/lib/octavia/123/haproxy.cfg.new', 'w')
             handle = m()
             handle.write.assert_called_once_with(six.b('test'))
             mock_subprocess.assert_called_once_with(
-                "haproxy -c -f /var/lib/octavia/123/haproxy.cfg.new".split(),
+                "haproxy -c -L {peer} -f {config_file}".format(
+                    config_file=file_name,
+                    peer=(octavia_utils.
+                          base64_sha1_string('amp_123').rstrip('='))).split(),
                 stderr=-2)
             mock_rename.assert_called_once_with(
                 '/var/lib/octavia/123/haproxy.cfg.new',
                 '/var/lib/octavia/123/haproxy.cfg')
 
         # exception writing
-        m = mock.Mock()
-        m.side_effect = Exception()  # open crashes
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        m = mock.mock_open()
+        m.side_effect = IOError()  # open crashes
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
-                              '/listeners/123/haproxy', data='test')
+                              '/listeners/amp_123/123/haproxy',
+                              data='test')
             self.assertEqual(500, rv.status_code)
 
         # check if files get created
         mock_exists.return_value = False
+        init_path = '/etc/init/haproxy-123.conf'
         m = mock.mock_open()
 
         # happy case upstart file exists
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
             rv = self.app.put('/' + api_server.VERSION +
-                              '/listeners/123/haproxy', data='test')
+                              '/listeners/amp_123/123/haproxy',
+                              data='test')
+
             self.assertEqual(202, rv.status_code)
-            m.assert_any_call('/var/lib/octavia/123/haproxy.cfg.new', 'w')
-            m.assert_any_call(util.UPSTART_DIR + '/haproxy-123.conf', 'w')
-            handle = m()
+            mode = (stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
+                    stat.S_IROTH | stat.S_IXOTH)
+            mock_open.assert_called_with(init_path, flags, mode)
+            mock_fdopen.assert_called_with(123, 'w')
+            handle = mock_fdopen()
             handle.write.assert_any_call(six.b('test'))
             # skip the template stuff
             mock_makedirs.assert_called_with('/var/lib/octavia/123')
@@ -96,25 +115,34 @@ class ServerTestCase(base.TestCase):
         mock_exists.return_value = True
         mock_subprocess.side_effect = [subprocess.CalledProcessError(
             7, 'test', RANDOM_ERROR)]
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
             rv = self.app.put('/' + api_server.VERSION +
-                              '/listeners/123/haproxy', data='test')
+                              '/listeners/amp_123/123/haproxy',
+                              data='test')
             self.assertEqual(400, rv.status_code)
             self.assertEqual(
                 {'message': 'Invalid request', u'details': u'random error'},
                 json.loads(rv.data.decode('utf-8')))
-            m.assert_called_with('/var/lib/octavia/123/haproxy.cfg.new', 'w')
-            handle = m()
+            mode = stat.S_IRUSR | stat.S_IWUSR
+            mock_open.assert_called_with(file_name, flags, mode)
+            mock_fdopen.assert_called_with(123, 'w')
+            handle = mock_fdopen()
             handle.write.assert_called_with(six.b('test'))
             mock_subprocess.assert_called_with(
-                "haproxy -c -f /var/lib/octavia/123/haproxy.cfg.new".split(),
+                "haproxy -c -L {peer} -f {config_file}".format(
+                    config_file=file_name,
+                    peer=(octavia_utils.
+                          base64_sha1_string('amp_123').rstrip('='))).split(),
                 stderr=-2)
-            mock_remove.assert_called_once_with(
-                '/var/lib/octavia/123/haproxy.cfg.new')
+            mock_remove.assert_called_once_with(file_name)
 
     @mock.patch('os.path.exists')
+    @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
+                'vrrp_check_script_update')
     @mock.patch('subprocess.check_output')
-    def test_start(self, mock_subprocess, mock_exists):
+    def test_start(self, mock_subprocess, mock_vrrp, mock_exists):
         rv = self.app.put('/' + api_server.VERSION + '/listeners/123/error')
         self.assertEqual(400, rv.status_code)
         self.assertEqual(
@@ -246,7 +274,7 @@ class ServerTestCase(base.TestCase):
         mock_exists.side_effect = [True]
         m = mock.mock_open(read_data=CONTENT)
 
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch.object(builtins, 'open', m):
             rv = self.app.get('/' + api_server.VERSION +
                               '/listeners/123/haproxy')
             self.assertEqual(200, rv.status_code)
@@ -288,7 +316,7 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(200, rv.status_code)
         self.assertEqual(
             [{'status': consts.ACTIVE, 'type': 'test', 'uuid': '123'},
-             {'status': consts.ERROR, 'type': 'http', 'uuid': '456'}],
+             {'status': consts.ERROR, 'type': '', 'uuid': '456'}],
             json.loads(rv.data.decode('utf-8')))
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
@@ -316,7 +344,7 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(200, rv.status_code)
         self.assertEqual(dict(
             status=consts.ERROR,
-            type='test',
+            type='',
             uuid='123'), json.loads(rv.data.decode('utf-8')))
 
         # Listener ACTIVE
@@ -378,12 +406,8 @@ class ServerTestCase(base.TestCase):
     @mock.patch('os.path.exists')
     def test_get_certificate_md5(self, mock_exists):
         CONTENT = "TestTest"
-        mock_exists.side_effect = [False]
-        rv = self.app.get('/' + api_server.VERSION +
-                          '/listeners/123/certificates/test.pem')
-        self.assertEqual(404, rv.status_code)
 
-        mock_exists.side_effect = [True, False]
+        mock_exists.side_effect = [False]
         rv = self.app.get('/' + api_server.VERSION +
                           '/listeners/123/certificates/test.pem')
         self.assertEqual(404, rv.status_code)
@@ -403,7 +427,7 @@ class ServerTestCase(base.TestCase):
         m = mock.mock_open(read_data=CONTENT)
         mock_exists.return_value = True
         mock_exists.side_effect = None
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch.object(builtins, 'open', m):
             rv = self.app.get('/' + api_server.VERSION +
                               '/listeners/123/certificates/test.pem')
         self.assertEqual(200, rv.status_code)
@@ -411,27 +435,19 @@ class ServerTestCase(base.TestCase):
                          json.loads(rv.data.decode('utf-8')))
 
     @mock.patch('os.path.exists')
-    @mock.patch('os.fchmod')
     @mock.patch('os.makedirs')
-    def test_upload_certificate_md5(self, mock_makedir, mock_chmod,
-                                    mock_exists):
-        mock_exists.side_effect = [False]
-        rv = self.app.put('/' + api_server.VERSION +
-                          '/listeners/123/certificates/test.pem',
-                          data='TestTest')
-        self.assertEqual(404, rv.status_code)
+    def test_upload_certificate_md5(self, mock_makedir, mock_exists):
 
         # wrong file name
-        mock_exists.side_effect = [True]
         rv = self.app.put('/' + api_server.VERSION +
                           '/listeners/123/certificates/test.bla',
                           data='TestTest')
         self.assertEqual(400, rv.status_code)
 
-        mock_exists.side_effect = [True, True, True]
+        mock_exists.return_value = True
         m = mock.mock_open()
 
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
                               '/listeners/123/certificates/test.pem',
                               data='TestTest')
@@ -439,26 +455,24 @@ class ServerTestCase(base.TestCase):
             self.assertEqual(OK, json.loads(rv.data.decode('utf-8')))
             handle = m()
             handle.write.assert_called_once_with(six.b('TestTest'))
-            mock_chmod.assert_called_once_with(handle.fileno(), 0o600)
 
-        mock_exists.side_effect = [True, False]
+        mock_exists.return_value = False
         m = mock.mock_open()
 
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
                               '/listeners/123/certificates/test.pem',
                               data='TestTest')
             self.assertEqual(200, rv.status_code)
             self.assertEqual(OK, json.loads(rv.data.decode('utf-8')))
             handle = m()
-            mock_makedir.called_once_with('/var/lib/octavia/123')
+            handle.write.assert_called_once_with(six.b('TestTest'))
+            mock_makedir.assert_called_once_with('/var/lib/octavia/certs/123')
 
-    @mock.patch('os.fchmod')
-    def test_upload_server_certificate(self, mock_chmod):
+    def test_upload_server_certificate(self):
         certificate_update.BUFFER = 5  # test the while loop
         m = mock.mock_open()
-
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
                               '/certificate',
                               data='TestTest')
@@ -467,14 +481,21 @@ class ServerTestCase(base.TestCase):
             handle = m()
             handle.write.assert_any_call(six.b('TestT'))
             handle.write.assert_any_call(six.b('est'))
-            mock_chmod.assert_called_once_with(handle.fileno(), 0o600)
 
     @mock.patch('netifaces.interfaces')
     @mock.patch('netifaces.ifaddresses')
+    @mock.patch('pyroute2.IPRoute')
+    @mock.patch('pyroute2.NetNS')
     @mock.patch('subprocess.check_output')
-    def test_plug_network(self, mock_check_output, mock_ifaddress,
-                          mock_interfaces):
+    def test_plug_network(self, mock_check_output, mock_netns,
+                          mock_pyroute2, mock_ifaddress, mock_interfaces):
         port_info = {'mac_address': '123'}
+        test_int_num = random.randint(0, 9999)
+
+        netns_handle = mock_netns.return_value.__enter__.return_value
+        netns_handle.get_links.return_value = [0] * test_int_num
+
+        test_int_num = str(test_int_num)
 
         # No interface at all
         mock_interfaces.side_effect = [[]]
@@ -500,21 +521,37 @@ class ServerTestCase(base.TestCase):
         mock_interfaces.side_effect = [['blah']]
         mock_ifaddress.side_effect = [[netifaces.AF_LINK],
                                       {netifaces.AF_LINK: [{'addr': '123'}]}]
-        m = mock.mock_open()
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = '/etc/netns/{0}/network/interfaces.d/eth{1}.cfg'.format(
+            consts.AMPHORA_NAMESPACE, test_int_num)
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
             rv = self.app.post('/' + api_server.VERSION + "/plug/network",
                                content_type='application/json',
                                data=json.dumps(port_info))
             self.assertEqual(202, rv.status_code)
-            m.assert_called_once_with(
-                '/etc/network/interfaces.d/blah.cfg', 'w')
+
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+
             handle = m()
-            handle.write.assert_called_once_with(
+            handle.write.assert_any_call(
                 '\n# Generated by Octavia agent\n'
-                'auto blah blah:0\n'
-                'iface blah inet dhcp')
+                'auto eth' + test_int_num + ' eth' + test_int_num +
+                ':0\niface eth' + test_int_num + ' inet dhcp')
             mock_check_output.assert_called_with(
-                ['ifup', 'blah'], stderr=-2)
+                ['ip', 'netns', 'exec', consts.AMPHORA_NAMESPACE,
+                 'ifup', 'eth' + test_int_num], stderr=-2)
 
         # same as above but ifup fails
         mock_interfaces.side_effect = [['blah']]
@@ -524,7 +561,7 @@ class ServerTestCase(base.TestCase):
             7, 'test', RANDOM_ERROR), subprocess.CalledProcessError(
             7, 'test', RANDOM_ERROR)]
         m = mock.mock_open()
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.post('/' + api_server.VERSION + "/plug/network",
                                content_type='application/json',
                                data=json.dumps(port_info))
@@ -536,10 +573,15 @@ class ServerTestCase(base.TestCase):
 
     @mock.patch('netifaces.interfaces')
     @mock.patch('netifaces.ifaddresses')
-    @mock.patch('subprocess.check_output')
     @mock.patch('pyroute2.IPRoute')
-    def test_plug_VIP(self, mock_pyroute2, mock_check_output, mock_ifaddress,
-                      mock_interfaces):
+    @mock.patch('pyroute2.netns.create')
+    @mock.patch('pyroute2.NetNS')
+    @mock.patch('subprocess.check_output')
+    @mock.patch('shutil.copytree')
+    @mock.patch('os.makedirs')
+    def test_plug_VIP(self, mock_makedirs, mock_copytree, mock_check_output,
+                      mock_netns, mock_netns_create, mock_pyroute2,
+                      mock_ifaddress, mock_interfaces):
 
         subnet_info = {'subnet_cidr': '10.0.0.0/24',
                        'gateway': '10.0.0.1',
@@ -578,17 +620,32 @@ class ServerTestCase(base.TestCase):
         mock_interfaces.side_effect = [['blah']]
         mock_ifaddress.side_effect = [[netifaces.AF_LINK],
                                       {netifaces.AF_LINK: [{'addr': '123'}]}]
-        m = mock.mock_open()
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = '/etc/netns/{}/network/interfaces.d/blah.cfg'.format(
+            consts.AMPHORA_NAMESPACE)
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
+
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
             rv = self.app.post('/' + api_server.VERSION +
                                "/plug/vip/203.0.113.2",
                                content_type='application/json',
                                data=json.dumps(subnet_info))
             self.assertEqual(202, rv.status_code)
-            m.assert_called_once_with(
-                '/etc/network/interfaces.d/blah.cfg', 'w')
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+
             handle = m()
-            handle.write.assert_called_once_with(
+            handle.write.assert_any_call(
                 '\n# Generated by Octavia agent\n'
                 'auto blah blah:0\n'
                 'iface blah inet dhcp\n'
@@ -597,7 +654,8 @@ class ServerTestCase(base.TestCase):
                 'broadcast 203.0.113.255\n'
                 'netmask 255.255.255.0')
             mock_check_output.assert_called_with(
-                ['ifup', 'blah:0'], stderr=-2)
+                ['ip', 'netns', 'exec', consts.AMPHORA_NAMESPACE,
+                 'ifup', 'blah:0'], stderr=-2)
 
         mock_interfaces.side_effect = [['blah']]
         mock_ifaddress.side_effect = [[netifaces.AF_LINK],
@@ -607,8 +665,9 @@ class ServerTestCase(base.TestCase):
             subprocess.CalledProcessError(
                 7, 'test', RANDOM_ERROR), subprocess.CalledProcessError(
                 7, 'test', RANDOM_ERROR)]
+
         m = mock.mock_open()
-        with mock.patch('%s.open' % BUILTINS, m, create=True):
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.post('/' + api_server.VERSION +
                                "/plug/vip/203.0.113.2",
                                content_type='application/json',
@@ -618,3 +677,99 @@ class ServerTestCase(base.TestCase):
                 {'details': RANDOM_ERROR,
                  'message': 'Error plugging VIP'},
                 json.loads(rv.data.decode('utf-8')))
+
+    @mock.patch('pyroute2.NetNS')
+    def test_get_interface(self, mock_netns):
+
+        netns_handle = mock_netns.return_value.__enter__.return_value
+
+        interface_res = {'interface': 'eth0'}
+
+        # Happy path
+        netns_handle.get_addr.return_value = [{
+            'index': 3, 'family': 2,
+            'attrs': [['IFA_ADDRESS', '203.0.113.2']]}]
+        netns_handle.get_links.return_value = [{
+            'attrs': [['IFLA_IFNAME', 'eth0']]}]
+        rv = self.app.get('/' + api_server.VERSION + '/interface/203.0.113.2',
+                          data=json.dumps(interface_res),
+                          content_type='application/json')
+        self.assertEqual(200, rv.status_code)
+
+        # Happy path with IPv6 address normalization
+        netns_handle.get_addr.return_value = [{
+            'index': 3, 'family': 10,
+            'attrs': [['IFA_ADDRESS',
+                       '0000:0000:0000:0000:0000:0000:0000:0001']]}]
+        netns_handle.get_links.return_value = [{
+            'attrs': [['IFLA_IFNAME', 'eth0']]}]
+        rv = self.app.get('/' + api_server.VERSION + '/interface/::1',
+                          data=json.dumps(interface_res),
+                          content_type='application/json')
+        self.assertEqual(200, rv.status_code)
+
+        # Nonexistent interface
+        rv = self.app.get('/' + api_server.VERSION + '/interface/10.0.0.1',
+                          data=json.dumps(interface_res),
+                          content_type='application/json')
+        self.assertEqual(404, rv.status_code)
+
+        # Invalid IP address
+        rv = self.app.get('/' + api_server.VERSION +
+                          '/interface/00:00:00:00:00:00',
+                          data=json.dumps(interface_res),
+                          content_type='application/json')
+        self.assertEqual(400, rv.status_code)
+
+    @mock.patch('os.path.exists')
+    @mock.patch('os.makedirs')
+    @mock.patch('os.rename')
+    @mock.patch('os.remove')
+    def test_upload_keepalived_config(self, mock_remove,
+                                      mock_rename, mock_makedirs, mock_exists):
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+
+        mock_exists.return_value = True
+        cfg_path = util.keepalived_cfg_path()
+        m = mock.mock_open()
+
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.put('/' + api_server.VERSION + '/vrrp/upload',
+                              data='test')
+
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_called_with(cfg_path, flags, mode)
+            mock_fdopen.assert_called_with(123, 'w')
+            self.assertEqual(200, rv.status_code)
+
+        mock_exists.return_value = False
+        script_path = util.keepalived_check_script_path()
+        m = mock.mock_open()
+
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.put('/' + api_server.VERSION + '/vrrp/upload',
+                              data='test')
+            mode = (stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
+                    stat.S_IROTH | stat.S_IXOTH)
+            mock_open.assert_called_with(script_path, flags, mode)
+            mock_fdopen.assert_called_with(123, 'w')
+            self.assertEqual(200, rv.status_code)
+
+    @mock.patch('subprocess.check_output')
+    def test_manage_service_vrrp(self, mock_check_output):
+        rv = self.app.put('/' + api_server.VERSION + '/vrrp/start')
+        self.assertEqual(202, rv.status_code)
+
+        rv = self.app.put('/' + api_server.VERSION + '/vrrp/restart')
+        self.assertEqual(400, rv.status_code)
+
+        mock_check_output.side_effect = subprocess.CalledProcessError(1,
+                                                                      'blah!')
+
+        rv = self.app.put('/' + api_server.VERSION + '/vrrp/start')
+        self.assertEqual(500, rv.status_code)
