@@ -18,6 +18,7 @@ from oslo_utils import uuidutils
 import requests_mock
 import six
 
+from octavia.amphorae.driver_exceptions import exceptions as driver_except
 from octavia.amphorae.drivers.haproxy import exceptions as exc
 from octavia.amphorae.drivers.haproxy import rest_api_driver as driver
 from octavia.db import models
@@ -173,6 +174,11 @@ class TestAmphoraAPIClientTest(base.TestCase):
         self.amp = models.Amphora(lb_network_ip='127.0.0.1', compute_id='123')
         self.port_info = dict(mac_address='123')
 
+    def test_request(self):
+        self.assertRaises(driver_except.TimeOutException,
+                          self.driver.request,
+                          'get', self.amp, 'unavailableURL')
+
     @requests_mock.mock()
     def test_get_info(self, m):
         info = {"hostname": "some_hostname", "version": "some_version",
