@@ -17,18 +17,14 @@ import binascii
 import random
 import socket
 
+import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
-import six
 
 from octavia.amphorae.drivers.health import heartbeat_udp
 from octavia.common import exceptions
 from octavia.tests.unit import base
 
-if six.PY2:
-    import mock
-else:
-    import unittest.mock as mock
 
 FAKE_ID = 1
 KEY = 'TEST'
@@ -39,13 +35,8 @@ RLIMIT = random.randrange(1, 100)
 
 class TestHeartbeatUDP(base.TestCase):
 
-    @mock.patch('socket.getaddrinfo')
-    @mock.patch('socket.socket')
-    def setUp(self, mock_socket, mock_getaddrinfo):
+    def setUp(self):
         super(TestHeartbeatUDP, self).setUp()
-        self.mock_socket = mock_socket
-        self.mock_getaddrinfo = mock_getaddrinfo
-        self.mock_getaddrinfo.return_value = [range(1, 6)]
         self.health_update = mock.Mock()
         self.stats_update = mock.Mock()
 
@@ -55,9 +46,6 @@ class TestHeartbeatUDP(base.TestCase):
         self.conf.config(group="health_manager", bind_port=PORT)
         self.conf.config(group="health_manager", sock_rlimit=0)
 
-        self.udp_status_getter = heartbeat_udp.UDPStatusGetter(
-            self.health_update, self.stats_update)
-
     @mock.patch('socket.getaddrinfo')
     @mock.patch('socket.socket')
     def test_update(self, mock_socket, mock_getaddrinfo):
@@ -70,9 +58,9 @@ class TestHeartbeatUDP(base.TestCase):
         getter = heartbeat_udp.UDPStatusGetter(
             None, None)
 
-        self.mock_getaddrinfo.assert_called_with(IP, PORT, 0, 2)
-        self.assertEqual(self.udp_status_getter.sockaddr, 5)
-        self.mock_socket.assert_called_with(1, socket.SOCK_DGRAM)
+        mock_getaddrinfo.assert_called_with(IP, PORT, 0, 2)
+        self.assertEqual(5, getter.sockaddr)
+        mock_socket.assert_called_with(1, socket.SOCK_DGRAM)
         bind_mock.assert_called_once_with((IP, PORT))
 
         self.conf.config(group="health_manager", sock_rlimit=RLIMIT)
@@ -98,15 +86,24 @@ class TestHeartbeatUDP(base.TestCase):
         bin_msg = binascii.unhexlify(sample_msg)
         recvfrom.return_value = bin_msg, 2
         (obj, srcaddr) = getter.dorecv()
-        self.assertEqual(srcaddr, 2)
-        self.assertEqual(obj, {"testkey": "TEST"})
+        self.assertEqual(2, srcaddr)
+        self.assertEqual({"testkey": "TEST"}, obj)
 
-    def test_check(self):
+    @mock.patch('socket.getaddrinfo')
+    @mock.patch('socket.socket')
+    def test_check(self, mock_socket, mock_getaddrinfo):
+        socket_mock = mock.MagicMock()
+        mock_socket.return_value = socket_mock
+        mock_getaddrinfo.return_value = [range(1, 6)]
         mock_dorecv = mock.Mock()
-        self.udp_status_getter.dorecv = mock_dorecv
+
+        getter = heartbeat_udp.UDPStatusGetter(
+            self.health_update, self.stats_update)
+        getter.dorecv = mock_dorecv
         mock_dorecv.side_effect = [(dict(id=FAKE_ID), 2)]
 
-        self.udp_status_getter.check()
+        getter.check()
+        getter.executor.shutdown()
         self.health_update.update_health.assert_called_once_with({'id': 1})
         self.stats_update.update_stats.assert_called_once_with({'id': 1})
 
@@ -118,13 +115,12 @@ class TestHeartbeatUDP(base.TestCase):
         self.mock_getaddrinfo.return_value = [range(1, 6)]
 
         mock_dorecv = mock.Mock()
-        self.udp_status_getter = heartbeat_udp.UDPStatusGetter(
-            None, None)
+        getter = heartbeat_udp.UDPStatusGetter(None, None)
 
-        self.udp_status_getter.dorecv = mock_dorecv
+        getter.dorecv = mock_dorecv
         mock_dorecv.side_effect = [(dict(id=FAKE_ID), 2)]
 
-        self.udp_status_getter.check()
+        getter.check()
 
     @mock.patch('socket.getaddrinfo')
     @mock.patch('socket.socket')
@@ -141,11 +137,10 @@ class TestHeartbeatUDP(base.TestCase):
         self.mock_getaddrinfo.return_value = [range(1, 6)]
 
         mock_dorecv = mock.Mock()
-        self.udp_status_getter = heartbeat_udp.UDPStatusGetter(
-            None, None)
+        getter = heartbeat_udp.UDPStatusGetter(None, None)
 
-        self.udp_status_getter.dorecv = mock_dorecv
+        getter.dorecv = mock_dorecv
         mock_dorecv.side_effect = exceptions.InvalidHMACException
 
-        self.udp_status_getter.check()
+        getter.check()
         self.assertFalse(mock_submit.called)
